use bevy::prelude::{Component, Reflect, Resource};
use fj_core::objects::{Sketch, Solid};
use fj_math::{Point, Vector};
use fj_interop::mesh::Mesh as FjMesh;
use fj_core::storage::Handle as FjHandle;

#[derive(Component)]
struct FjSolidWrapper {
    handle: fj_core::storage::Handle<Solid>,
}

#[derive(Component, Debug)]
pub struct FjMeshWrapper {
    pub mesh: FjMesh<Point<3>>,
    pub handle: FjHandle<Solid>,
    pub sketch: Sketch,
}

pub(crate) trait ToPoint3 {
    fn to_point3(&self) -> Point<3>;
}


impl ToPoint3 for bevy::prelude::Vec3 {
    fn to_point3(&self) -> fj_math::Point<3> {
        fj_math::Point { coords: Vector { components: [fj_math::Scalar::from_u64(0),fj_math::Scalar::from_u64(0),fj_math::Scalar::from_u64(0)] } }
    }
}

pub(crate) trait ToVec3 {
    fn to_vec3(&self) -> bevy::prelude::Vec3;
}

impl ToVec3 for fj_math::Point<3> {
    fn to_vec3(&self) -> bevy::prelude::Vec3 {
        bevy::prelude::Vec3::new(self.x.into_f32(), self.y.into_f32(), self.z.into_f32())
    }
}

impl ToVec3 for fj_math::Vector<3> {
    fn to_vec3(&self) -> bevy::prelude::Vec3 {
        bevy::prelude::Vec3::new(self.x.into_f32(), self.y.into_f32(), self.z.into_f32())
    }
}

pub(crate) trait ToVec2 {
    fn to_vec2(&self) -> bevy::prelude::Vec2;
}

impl crate::interop::ToVec2 for fj_math::Point<2> {
    fn to_vec2(&self) -> bevy::prelude::Vec2 {
        bevy::prelude::Vec2::new(self.u.into_f32(), self.v.into_f32())
    }
}

impl crate::interop::ToVec2 for fj_math::Vector<2> {
    fn to_vec2(&self) -> bevy::prelude::Vec2 {
        bevy::prelude::Vec2::new(self.u.into_f32(), self.v.into_f32())
    }
}
